/*
 * calcget - a command line utility to save a variable from
 * a TI89 Titanium calculator over the USB link
 *
 * Copyright (C) 2010 Michael Jordan
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "ti_usb.h"
#include "ti_file.h"

int main(int argc, char *argv[]) {
	if(argc > 1) {
		string argument;
		argument.append(argv[1]);
		bool help = false;
		bool valid = false;
		if((argument == "-h") || (argument == "--help")) {
			valid = true;
			help = true;
		}
		if(!valid) help = true;
		if(help) {
			cout <<
					"\ncalcget is a utility to save individual files from a\n" <<
					"TI89 Titanium calculator over the USB link.\n\n" << flush;
			cout <<
					"It will get a listing of the variables on the calculator,\n" <<
					"and then allow the user to choose which ones to save.  The\n" <<
					"files are saved in standard TI format so that they can be\n" <<
					"reloaded using TI Connect.\n" << flush;
			cout <<
					"Keep in mind that this is, in fact, free software, developed\n" <<
					"independently and therefore comes with no warranty.\n" << flush;
			return EX_OK;
		}
	}
	cout << "Searching for calculator..." << flush;
	ti_usb calc;

	// Calling is_good() here verifies that there was a calculator located on
	// the USB bus and that we can in fact communicate with it.
	if(calc.is_good()) {
		vector<ti_usb::ls_item_info> items;
		ti_usb::ls_item_info ls_item;
		bool test = true;;

		while (test) {

			// Get a listing of vars from the calculator and print it out so the
			// user can choose which one they wish to back up.
			cout << "Calculator located...Getting listing...\n";
			calc.ls(items);
			cout << right << setw(13) << "Folder:" <<
				left << setw(8) << "Name" <<
				right << setw(10) << "Size:" << "  Archived Locked Type" <<
				endl << endl;
			for(int i = 0; i < (int)items.size(); i++) {
				if(i > 0) cout << right << setw(4) << i;
				else cout << right << setw(4) << " ";
				cout << right << setw(8) << items[i].parent_folder << ":" <<
					left << setw(8) << items[i].name <<
					right << setw(10) << items[i].size << "  " <<
					boolalpha << left << setw(8) << items[i].archived << " " <<
					items[i].locked << "\t " <<
					calc.typeid_to_string(items[i].type) << endl;
			}
			cout << endl;
			cout << "Enter an item number (enter an item NOT on the list to quit):  ";
			int num;
			cin >> num;
			test = ((num >= 1) && (num < (int)items.size()));
			if(test) {

				// Print the individual item out for the user to see.
				cout << right << setw(13) << "Folder:" <<
						left << setw(8) << "Name" <<
						right << setw(10) << "Size:" << " Archived Locked Type" << endl;
				cout << right << setw(4) << num <<
						right << setw(8) << items[num].parent_folder << ":" <<
						left << setw(8) << items[num].name <<
						right << setw(10) << items[num].size << " " <<
						boolalpha << left << setw(8) << items[num].archived << " " <<
						items[num].locked << "\t" <<
						calc.typeid_to_string(items[num].type) << endl << endl;
				ls_item = items[num];
				items.clear();
				unsigned char buf[ls_item.size];

				// Get the var from the calculator.
				if(calc.get_var_data(ls_item, buf) == 0) cout << "Success\n";
				else {
					cout << "Failed - aborting\n";
					return -1;
				}

				// Save the var to a file in the current directory.  The file is
				// saved using TI's naming scheme (i.e. a BASIC program called
				// simp in folder main will be saved as main.simp.89p, etc.).
				if(calc.var_save(ls_item, buf, "./") == 0) return 0;
				return -1;
			}
		}
		return 0;
	} cout << "No calculator found on the USB bus\n";
}
